import grpc, logging
from common.tools.client.RetryDecorator import retry, delay_exponential
from opticalattackmitigator.proto.optical_attack_mitigator_pb2 import AttackDescription, AttackResponse
from opticalattackmitigator.proto.optical_attack_mitigator_pb2_grpc import AttackMitigatorStub

LOGGER = logging.getLogger(__name__)
MAX_RETRIES = 15
DELAY_FUNCTION = delay_exponential(initial=0.01, increment=2.0, maximum=5.0)
RETRY_DECORATOR = retry(max_retries=MAX_RETRIES, delay_function=DELAY_FUNCTION, prepare_method_name='connect')

class OpticalAttackMitigatorClient:
    def __init__(self, address, port):
        self.endpoint = '{:s}:{:s}'.format(str(address), str(port))
        LOGGER.debug('Creating channel to {:s}...'.format(str(self.endpoint)))
        self.channel = None
        self.stub = None
        self.connect()
        LOGGER.debug('Channel created')

    def connect(self):
        self.channel = grpc.insecure_channel(self.endpoint)
        self.stub = AttackMitigatorStub(self.channel)

    def close(self):
        if(self.channel is not None): self.channel.close()
        self.channel = None
        self.stub = None

    @RETRY_DECORATOR
    def NotifyAttack(self, request : AttackDescription) -> AttackResponse:
        LOGGER.debug('NotifyAttack request: {:s}'.format(str(request)))
        response = self.stub.NotifyAttack(request)
        LOGGER.debug('NotifyAttack result: {:s}'.format(str(response)))
        return response
